﻿Type=Class
Version=6
ModulesStructureVersion=1
B4A=true
@EndOfDesignText@
'ByteBuffer Class module
Private Sub Class_Globals
	Private internalByteBuffer As JavaObject
	Public Const DOUBLE_SIZE As Int = 8
	Public Const FLOAT_SIZE As Int = 4
	Public Const INT_SIZE As Int = 4
	Public Const LONG_SIZE As Int = 8
	Public Const SHORT_SIZE As Int = 2
	Public Const LITTLE_ENDIAN As Int = 0
	Public Const BIG_ENDIAN As Int = 1
	Public Const TYPENAME As String = "java.nio.ByteBuffer"
End Sub

'This will initialize this ByteBuffer class and allocate the given number of bytes in the 
'capacity argument to it. If the value in capacity is negativ no allocation will be executed
'and it must be allocated with allocate or wrapped with wrap by yourself.
Public Sub Initialize(capacity As Int)
	If capacity >= 0 Then
		Allocate(capacity)
	End If
End Sub

'This will allocate the given number of bytes in the capacity argument and create a new ByteBuffer
'out of it. The old ByteBuffer will get lost.
Public Sub Allocate(capacity As Int)
	internalByteBuffer.InitializeStatic(TYPENAME)
	internalByteBuffer = internalByteBuffer.RunMethodJO("allocate", Array As Object(capacity))
End Sub

Public Sub getArray As Byte()
	Return internalByteBuffer.RunMethod("array", Null)
End Sub

Public Sub getCapacity As Int
	Return internalByteBuffer.RunMethod("capacity", Null)
End Sub

Public Sub Clear
	internalByteBuffer.RunMethod("clear", Null)
End Sub

Public Sub Get As Byte
	Return internalByteBuffer.RunMethod("get", Null)
End Sub

Public Sub Get2(index As Int) As Byte
	Return internalByteBuffer.RunMethod("get", Array As Object(index))
End Sub

Public Sub Get3(buffer() As Byte, offset As Int, count As Int)
	internalByteBuffer.RunMethod("get", Array As Object(buffer, offset, count))
End Sub

Public Sub Get4(index As Int, buffer() As Byte, offset As Int, count As Int)
	Dim oldPosition As Int = getPosition
	setPosition(index)
	internalByteBuffer.RunMethod("get", Array As Object(buffer, offset, count))
	setPosition(oldPosition)
End Sub

Public Sub GetDouble As Double
	Return internalByteBuffer.RunMethod("getDouble", Null)
End Sub

Public Sub GetDouble2(index As Int) As Double
	Return internalByteBuffer.RunMethod("getDouble", Array As Object(index))
End Sub

Public Sub GetFloat As Float
	Return internalByteBuffer.RunMethod("getFloat", Null)
End Sub

Public Sub GetFloat2(index As Int) As Float
	Return internalByteBuffer.RunMethod("getFloat", Array As Object(index))
End Sub

Public Sub GetInt As Int
	Return internalByteBuffer.RunMethod("getInt", Null)
End Sub

Public Sub GetInt2(index As Int) As Int
	Return internalByteBuffer.RunMethod("getInt", Array As Object(index))
End Sub

Public Sub GetLong As Long
	Return internalByteBuffer.RunMethod("getLong", Null)
End Sub

Public Sub GetLong2(index As Int) As Long
	Return internalByteBuffer.RunMethod("getLong", Array As Object(index))
End Sub

Public Sub GetShort As Short
	Return internalByteBuffer.RunMethod("getShort", Null)
End Sub

Public Sub GetShort2(index As Int) As Short
	Return internalByteBuffer.RunMethod("getShort", Array As Object(index))
End Sub

Public Sub getOrder As Int
	Dim orderResult As Int
	Dim byteOrder As Object = internalByteBuffer.RunMethod("order", Null)
	Dim byteOrderObject As JavaObject
	byteOrderObject.InitializeStatic("java.nio.ByteOrder")
	If byteOrder = byteOrderObject.GetField("LITTLE_ENDIAN") Then
		orderResult = LITTLE_ENDIAN
	Else if byteOrder = byteOrderObject.GetField("BIG_ENDIAN") Then
		orderResult = BIG_ENDIAN
	End If
	Return orderResult
End Sub

Public Sub setOrder(byteOrder As Int)
	Dim byteOrderObject As JavaObject
	byteOrderObject.InitializeStatic("java.nio.ByteOrder")
	Dim byteOrderField As Object
	If byteOrder = LITTLE_ENDIAN Then
		byteOrderField = byteOrderObject.GetField("LITTLE_ENDIAN")
	Else if byteOrder = BIG_ENDIAN Then
		byteOrderField = byteOrderObject.GetField("BIG_ENDIAN")
	Else
		Dim ex As ExceptionEx
		ex.Initialize("ByteBuffer->setOrder: Argument out of Range!")
		ex.Throw
	End If
	internalByteBuffer.RunMethod("order", Array As Object(byteOrderField))	
End Sub

Public Sub getPosition As Int
	Return internalByteBuffer.RunMethod("position", Null)
End Sub

Public Sub setPosition(newPosition As Int)
	internalByteBuffer.RunMethod("position", Array As Object(newPosition))
End Sub

Public Sub Put(value As Byte)
	internalByteBuffer.RunMethod("put", Array As Object(value))
End Sub

Public Sub Put2(index As Int, value As Byte)
	internalByteBuffer.RunMethod("put", Array As Object(index, value))
End Sub

Public Sub PutBuffer(buffer() As Byte)
	internalByteBuffer.RunMethod("put", Array As Object(buffer))
End Sub

Public Sub PutBuffer2(buffer() As Byte, offset As Int, length As Int)
	internalByteBuffer.RunMethod("put", Array As Object(buffer, offset, length))
End Sub

Public Sub PutDouble(value As Double)
	internalByteBuffer.RunMethod("putDouble", Array As Object(value))
End Sub

Public Sub PutDouble2(index As Int, value As Double)
	internalByteBuffer.RunMethod("putDouble", Array As Object(index, value))
End Sub

Public Sub PutFloat(value As Float)
	internalByteBuffer.RunMethod("putFloat", Array As Object(value))
End Sub

Public Sub PutFloat2(index As Int, value As Float)
	internalByteBuffer.RunMethod("putFloat", Array As Object(index, value))
End Sub

Public Sub PutInt(value As Int)
	internalByteBuffer.RunMethod("putInt", Array As Object(value))
End Sub

Public Sub PutInt2(index As Int, value As Int)
	internalByteBuffer.RunMethod("putInt", Array As Object(index, value))
End Sub

Public Sub PutLong(value As Long)
	internalByteBuffer.RunMethod("putLong", Array As Object(value))
End Sub

Public Sub PutLong2(index As Int, value As Long)
	internalByteBuffer.RunMethod("putLong", Array As Object(index, value))
End Sub

Public Sub PutShort(value As Short)
	internalByteBuffer.RunMethod("putShort", Array As Object(value))
End Sub

Public Sub PutShort2(index As Int, value As Short)
	internalByteBuffer.RunMethod("putShort", Array As Object(index, value))
End Sub

Public Sub getRemaining As Int
	Return internalByteBuffer.RunMethod("remaining", Null)
End Sub

Public Sub ToByteArray As Byte()
	Dim buffer() As Byte = internalByteBuffer.RunMethod("array", Null)
	Dim resultBuffer(buffer.Length) As Byte
	Dim systemObject As JavaObject
	systemObject.InitializeStatic("java.lang.System")
	systemObject.RunMethod("arraycopy", Array As Object(buffer, 0, resultBuffer, 0, resultBuffer.Length))
	Return resultBuffer
End Sub

Public Sub Wrap(buffer() As Byte)
	Dim byteBufferObject As JavaObject
	byteBufferObject.InitializeStatic(TYPENAME)
	internalByteBuffer = byteBufferObject.RunMethod("wrap", Array As Object(buffer))
End Sub

Public Sub Wrap2(buffer() As Byte, start As Int, byteCount As Int)
	Dim byteBufferObject As JavaObject
	byteBufferObject.InitializeStatic(TYPENAME)
	internalByteBuffer = byteBufferObject.RunMethod("wrap", Array As Object(buffer, start, byteCount))
End Sub