﻿Type=Class
Version=6
ModulesStructureVersion=1
B4A=true
@EndOfDesignText@
'ByteStreamWriter Class
Private Sub Class_Globals
	Private internalOutputStream As OutputStream
	Private internalByteBuffer As ByteBuffer 
End Sub

Public Sub Initialize(outStream As OutputStream)
	internalOutputStream = outStream
	internalByteBuffer.Initialize(-1)
	internalByteBuffer.Allocate(internalByteBuffer.LONG_SIZE)
	internalByteBuffer.Order = internalByteBuffer.LITTLE_ENDIAN
End Sub

Public Sub Close
	internalByteBuffer.Clear
	internalOutputStream.Flush
	internalOutputStream.Close
End Sub

Public Sub WriteBitmap(value As Bitmap)
	WriteBitmap2(value, "PNG", 100)
End Sub

Public Sub WriteBitmap2(value As Bitmap, format As String, quality As Int)
	Dim buffer() As Byte
	Dim memoryOutStream As OutputStream
	memoryOutStream.InitializeToBytesArray(1)
	value.WriteToStream(memoryOutStream, quality, format)
	buffer = memoryOutStream.ToBytesArray
	memoryOutStream.Close
	WriteInt(buffer.Length)
	WriteBuffer(buffer)
End Sub

Public Sub WriteBoolean(value As Boolean)
	Dim buffer(1) As Byte
	If value = True Then
		buffer(0) = 1
	Else
		buffer(0) = 0
	End If
	WriteBuffer(buffer)
End Sub

Public Sub WriteBuffer(buffer() As Byte)
	internalOutputStream.WriteBytes(buffer, 0, buffer.Length)
End Sub

Public Sub WriteBuffer2(buffer() As Byte, count As Int)
	internalOutputStream.WriteBytes(buffer, 0, count)
End Sub

Public Sub WriteBuffer3(buffer() As Byte, offset As Int, count As Int)
	internalOutputStream.WriteBytes(buffer, offset, count)
End Sub

Public Sub WriteDouble(value As Double)
	Dim buffer() As Byte
	internalByteBuffer.PutDouble2(0, value)
	buffer = internalByteBuffer.Array
	WriteBuffer2(buffer, internalByteBuffer.DOUBLE_SIZE)
End Sub

Public Sub WriteFloat(value As Float)
	Dim buffer() As Byte
	internalByteBuffer.PutFloat2(0, value)
	buffer = internalByteBuffer.Array
	WriteBuffer2(buffer, internalByteBuffer.FLOAT_SIZE)
End Sub

Public Sub WriteInt(value As Int)
	Dim buffer() As Byte
	internalByteBuffer.PutInt2(0, value)
	buffer = internalByteBuffer.Array
	WriteBuffer2(buffer, internalByteBuffer.INT_SIZE)
End Sub

Public Sub WriteLong(value As Long)
	Dim buffer() As Byte
	internalByteBuffer.PutLong2(0, value)
	buffer = internalByteBuffer.Array
	WriteBuffer2(buffer, internalByteBuffer.LONG_SIZE)
End Sub

Public Sub WriteString(value As String)
	Dim buffer() As Byte
	buffer = value.GetBytes("UTF8")
	WriteInt(buffer.Length)
	If buffer.Length > 0 Then
		WriteBuffer(buffer)
	End If
End Sub

Public Sub WriteObject(value As Object)
	If value Is Int Then
		WriteInt(value)
	Else If value Is String Then
		WriteString(value)
	Else If value Is Boolean Then
		WriteBoolean(value)
	Else If value Is Long Then
		WriteLong(value)
	Else If value Is Float Then
		WriteFloat(value)
	Else If value Is Double Then
		WriteDouble(value)
	Else If value Is Bitmap Then
		WriteBitmap(value)
	Else
		Log("ByteStreamWriter->WriteObject can not found the right write method for this value: " & value)
	End If
End Sub