﻿Type=Class
Version=6
ModulesStructureVersion=1
B4A=true
@EndOfDesignText@
#IgnoreWarnings: 30

'ContactBook Class module
Private Sub Class_Globals
	Private internalContactHandler As ContactHandler
End Sub

'This will initialize this contactbook object
Public Sub Initialize
	internalContactHandler.Initialize
End Sub

'This will add a new contact to the device contacts and return them as contactitem class.
Public Sub AddContact(displayName As String) As ContactItem
	Dim contactID As Long = internalContactHandler.CreateContact(displayName)
	Dim resultContact As ContactItem
	resultContact.Initialize(contactID, displayName, internalContactHandler)
	Return resultContact
End Sub

'This will get all contacts from the device and return them as list filled with contactitem classes.
'visibleOnly - set it to True to get only visible contacts, otherwise set it to False.
Public Sub GetAllContacts(visibleOnly As Boolean) As List
	Dim idNameMap As Map = internalContactHandler.GetContactsFromData(ContactMimeTypes.NAME, "data1", "null", "<>", True, visibleOnly)
	Return MapToContactList(idNameMap)
End Sub

'This will get all contacts from the device and return them as list filled with contactitem classes.
'visibleOnly - set it to True to get only visible contacts, otherwise set it to False.
'sorted - set it to True to get a sorted list by displayname, otherwise set it to False.
Public Sub GetAllContacts2(visibleOnly As Boolean, sorted As Boolean) As List
	Dim idNameMap As Map = internalContactHandler.GetContactsFromData(ContactMimeTypes.NAME, "data1", "null", "<>", True, visibleOnly)
	If sorted Then
		Dim sortList As List
		sortList.Initialize
		For index = 0 To idNameMap.Size - 1
			sortList.Add(idNameMap.GetValueAt(index) & "," & index)
		Next
		sortList.Sort(True)
		Dim sortedIdNameMap As Map
		sortedIdNameMap.Initialize
		For index = 0 To sortList.Size - 1
			Dim value As String = sortList.Get(index)
			Dim signPos As Int = value.LastIndexOf(",")
			Dim mapIndex As Int = value.SubString(signPos + 1)
			sortedIdNameMap.Put(idNameMap.GetKeyAt(mapIndex), idNameMap.GetValueAt(mapIndex))
		Next
		idNameMap.Clear
		idNameMap = sortedIdNameMap
	End If
	Return MapToContactList(idNameMap)
End Sub

'This will get all contact groups
Public Sub GetAllGroups As ContactGroup()
	Dim idTitleMap As Map = internalContactHandler.GetAllGroups
	Dim resultGroups(idTitleMap.Size) As ContactGroup
	For index = 0 To resultGroups.Length - 1
		Dim group As ContactGroup
		group.Initialize(idTitleMap.GetKeyAt(index), idTitleMap.GetValueAt(index))
		'group.ID = 
		'group.Title = 
		resultGroups(index) = group
	Next
	idTitleMap.Clear
	Return resultGroups
End Sub

'This will get all contacts with the given display name as contactitem class list from the device.
'displayName = A part or the complete display name of the desired contact
'exact = True to search for the exact given diplay name, otherwise set it to False.
'visibleOnly = True for only visible contact results, otherwise False.
Public Sub GetContactsByName(displayName As String, exact As Boolean, visibleOnly As Boolean) As List
	Dim idNameMap As Map = internalContactHandler.GetContactsFromData(ContactMimeTypes.NAME, "data1", displayName, "=", exact, visibleOnly)
	Return MapToContactList(idNameMap)
End Sub

'This will get all contacts with the given group id as contactitem class list from the device.
'groupID = The group id
'visibleOnly = True for only visible contact results, otherwise False.
Public Sub GetContactsByGroupID(groupID As Long, visibleOnly As Boolean) As List
	Dim idNameMap As Map = internalContactHandler.GetContactsFromData(ContactMimeTypes.GROUP_MEMBERSHIP, "data1", groupID, "=", True, visibleOnly)
	Return MapToContactList(idNameMap)
End Sub

'This will get all contacts with the given phone number as contactitem class list from the device.
Public Sub GetContactsByPhoneNumber(phoneNumber As String, exact As Boolean, visibleOnly As Boolean) As List
	Dim idNameMap As Map = internalContactHandler.GetContactsFromData(ContactMimeTypes.PHONE, "data1", phoneNumber, "=", exact, visibleOnly)
	Return MapToContactList(idNameMap)
End Sub

'This will get all contacts that was or wasn't starred as contactitem class list from the device.
'starred = True to get only starred contact results, otherwise False.
'visibleOnly = True for only visible contact results, otherwise False.
Public Sub GetContactsByStarred(starred As Boolean, visibleOnly As Boolean) As List
	Dim starredValue As String
	If starred Then
		starredValue = "1"
	Else
		starredValue = "0"
	End If
	Dim idNameMap As Map = internalContactHandler.GetContactsFromData(ContactMimeTypes.NAME, "starred", starredValue, _
																	 "=", True, visibleOnly)
	Return MapToContactList(idNameMap)
End Sub

'This will get all contacts that have a thumbnail photo as contactitem class list from the device.
'visibleOnly = True for only visible contact results, otherwise False.
Public Sub GetContactsWithThumbnailPhoto(visibleOnly As Boolean) As List
	Dim idNameMap As Map = internalContactHandler.GetContactsFromData(ContactMimeTypes.PHOTO, "data15", "null", "<>", _
																	  True, visibleOnly)
	Return MapToContactList(idNameMap)
End Sub

'This will get the max display photo dimension value (width and height).
Public Sub getMaxDisplayPhotoDim As Int
	Return internalContactHandler.MaxDisplayPhotoDimensions
End Sub

'This will get the max thumbnail photo dimension value (width and height).
Public Sub getMaxThumbnailPhotoDim As Int
	Return internalContactHandler.MaxThumbnailPhotoDimensions
End Sub

'This will remove the given contact from the device.
'contact = An existing contact item of the device.
Public Sub Remove(contact As ContactItem)
	internalContactHandler.DeleteContact(contact.ID)
End Sub

'Tool function to create a list of contacts from a map.
Private Sub MapToContactList(idNameMap As Map) As List
	Dim resultContactList As List
	resultContactList.Initialize
	If idNameMap.IsInitialized And idNameMap.Size > 0 Then
		For index = 0 To idNameMap.Size - 1
			Dim foundContact As ContactItem
			foundContact.Initialize(idNameMap.GetKeyAt(index), idNameMap.GetValueAt(index), internalContactHandler)
			resultContactList.Add(foundContact)
		Next
	End If
	idNameMap.Clear
	Return resultContactList
End Sub