﻿Type=Class
Version=6
ModulesStructureVersion=1
B4A=true
@EndOfDesignText@
'ContactHandler Class module 
'FOR INTERNAL USE BY THE CLASSES ContactBook AND ContactItem ONLY!!!
Private Sub Class_Globals
	Private internalContactsUri As Uri
	Private internalDataUri As Uri
	'Private internalGroupsUri As Uri
	Private internalRawContactsUri As Uri
	Private internalContentResolver As ContentResolver
	
	Private internalDisplayPhotoDimensions As Int
	Private internalThumbnailPhotoDimensions As Int
End Sub

'This is for internal use only.
Public Sub Initialize
	internalContactsUri.Parse(ContactStatics.CONTACTS_URL)
	internalDataUri.Parse(ContactStatics.DATA_URL)
	'internalGroupsUri.Parse(ContactStatics.GROUPS_URL)
	internalRawContactsUri.Parse(ContactStatics.RAW_CONTACTS_URL)
	internalContentResolver.Initialize("ContentResolver")
	
	Dim displayPhotoClass As JavaObject
	displayPhotoClass.InitializeStatic("android.provider.ContactsContract.DisplayPhoto")
	Dim maxDimensions As Uri = displayPhotoClass.GetField("CONTENT_MAX_DIMENSIONS_URI")
	Dim dimensionsCursor As Cursor = internalContentResolver.Query(maxDimensions, Array As String("display_max_dim", _
																	"thumbnail_max_dim"), "", Null, "")
	dimensionsCursor.Position = 0
	internalDisplayPhotoDimensions = dimensionsCursor.GetString("display_max_dim")
	internalThumbnailPhotoDimensions = dimensionsCursor.GetString("thumbnail_max_dim")
	dimensionsCursor.Close
End Sub

'This will create and insert a contact with the given display name to the device and return the contact id as long value.
'displayName = The display name of the contact to add. 
Public Sub CreateContact(displayName As String) As Long
	Dim values As ContentValues
	values.Initialize
	values.PutNull("account_name")
	values.PutNull("account_type")
	Dim rawContactUri As Uri = internalContentResolver.Insert(internalRawContactsUri, values)
	Dim rawContactID As Long = rawContactUri.ParseId
	values.Initialize
	values.PutLong("raw_contact_id", rawContactID)
	values.PutString("mimetype", ContactMimeTypes.NAME)
	values.PutString("data1", displayName)
	internalContentResolver.Insert(internalDataUri, values)
	Dim contactCursor As Cursor = internalContentResolver.Query(internalDataUri, Array As String("contact_id"), _
																"raw_contact_id = ?", Array As String(rawContactID), "")
	contactCursor.Position = 0
	Dim contactID As Long = contactCursor.GetLong("contact_id")
	contactCursor.Close
	Return contactID
End Sub

'This will create a string array with count of "data" strings e.g.: "data1", "data2" ...
Public Sub CreateDataArray(count As Int) As String()
	Dim result(count) As String
	For index = 1 To count
		result(index - 1) = "data" & index
	Next
	Return result
End Sub

'This will delete/remove a contact from the device by the contacts id.
'contactID = The exact contacts id as long value.
Public Sub DeleteContact(contactID As Long)
	Dim contactCursor As Cursor = internalContentResolver.Query(internalRawContactsUri, Array As String("_id"), _
	 															"contact_id = ?", Array As String(contactID), "")
	If contactCursor.RowCount = 1 Then
		contactCursor.Position = 0
		Dim rawContactId As Long = contactCursor.GetLong("_id")
		contactCursor.Close
		internalContentResolver.Delete(internalRawContactsUri, "_id = ?", Array As String(rawContactId))
	Else
		Log("Error: More then one raw contacts found!")
		contactCursor.Close
	End If
End Sub

'This will delete the data that was exactly found by the given mime type and data values.
'convertExact = True to convert all dataValues exactly, otherwise False to ignore "null" and empty values.
'(e.g. "null" = NULL or "" = "" otherwise "null" = ignored or "" = ignored) 
Public Sub DeleteData(contactID As Long, mimeType As String, dataValues() As Object, convertExact As Boolean)
	Dim selectionItems As List
	Dim whereBuilder As StringBuilder
	selectionItems.Initialize
	selectionItems.Add(mimeType)
	selectionItems.Add(contactID)
	whereBuilder.Initialize
	whereBuilder.Append("mimetype = ? AND contact_id = ?")
	If dataValues <> Null And dataValues.Length > 0 Then
		For index = 1 To dataValues.Length
			Dim addValue As Boolean = False
			Dim dataValue As String = dataValues(index - 1)
			If dataValue.Length > 0 Then
				If dataValue.EqualsIgnoreCase("null") = False Then
					addValue = True
				End If
			Else
				If convertExact Then
					addValue = True
				End If
			End If
			If addValue Then
				selectionItems.Add(dataValue)
				whereBuilder.Append(" AND data").Append(index).Append(" = ?")
			End If
		Next
	End If
	Dim selectionArgs(selectionItems.Size) As String
	For index = 0 To selectionItems.Size - 1
		selectionArgs(index) = selectionItems.Get(index)
	Next
	selectionItems.Clear
	internalContentResolver.Delete(internalDataUri, whereBuilder.ToString, selectionArgs)
End Sub

'This will get all contact groups and retrun them as map.
Public Sub GetAllGroups As Map
	Dim groupsCursor As Cursor
	'groupsCursor = internalContentResolver.Query(internalGroupsUri, Array As String("_id", "title"), "title <> ?", _
	'											Array As String("null"), "")
	Dim jObj As JavaObject
	jObj.InitializeStatic("android.provider.ContactsContract.Groups")
	Dim contactGroupsUri As Object = jObj.GetField("CONTENT_SUMMARY_URI")
	groupsCursor = internalContentResolver.Query(contactGroupsUri, Array As String("_id", "title", "notes"), "", _
												Null, "")
	Dim groups As Map
	groups.Initialize
	For index = 0 To groupsCursor.RowCount - 1
		groupsCursor.Position = index
		Dim groupID As String = groupsCursor.GetLong("_id")
		If groups.ContainsKey(groupID) = False Then
			'Dim notes As String = groupsCursor.GetString("notes")
			groups.Put(groupID, groupsCursor.GetString("title"))
		End If
	Next
	Return groups
End Sub

Public Sub GetContactsFromData(mimeType As String, dataColumn As String, value As String, operator As String, _
	exact As Boolean, visibleOnly As Boolean) As Map
	If Not(exact) Then 
		operator = "LIKE"
		value = "%" & value & "%"
	End If
	Dim selection As String = "mimetype = ? AND " & dataColumn & " " & operator & " ? "
	If visibleOnly Then selection = selection & " AND in_visible_group = 1"
	Dim contentCursor As Cursor = internalContentResolver.Query(internalDataUri, Array As String("contact_id", _ 
													"display_name"), selection, Array As String(mimeType, value), "")
	Dim resultMap As Map
	resultMap.Initialize
	Dim id As String
	Dim displayName As String
	For index = 0 To contentCursor.RowCount - 1
		contentCursor.Position = index
		id = contentCursor.GetLong("contact_id")
		displayName = contentCursor.GetString("display_name")
		If resultMap.ContainsKey(id) Then 
			Continue
		End If
		resultMap.Put(id, displayName)
	Next
	contentCursor.Close
	Return resultMap
End Sub

Public Sub GetData(contactID As Long, mimeType As String, dataColumns() As String, blobs() As Boolean) As List
	Dim contentCursor As Cursor = internalContentResolver.Query(internalDataUri, dataColumns, "mimetype = ? AND contact_id = ?", _
														Array As String(mimeType, contactID), "")
	Dim results As List
	results.Initialize
	For index = 0 To contentCursor.RowCount - 1
		contentCursor.Position = index
		Dim row(dataColumns.Length) As Object
		For index2 = 0 To dataColumns.Length - 1
			If  blobs <> Null And blobs(index2) = True Then
				row(index2) = contentCursor.GetBlob2(index2)
			Else
				row(index2) = contentCursor.GetString2(index2)
				Dim data As String = row(index2)
				
				'For now convert all null strings into empty strings
				If data.Length > 0 And data = "null" Then
					row(index2) = ""
				End If
			End If
		Next
		results.Add(row)
	Next
	contentCursor.Close
	Return results
End Sub

'This will return the display photo uri as string of the contact found by the given contact id.
Public Sub GetPhotoUri(contactID As Long) As String
	Dim contentCursor As Cursor = internalContentResolver.Query(internalDataUri, Array As String("photo_uri"), _
																"contact_id = ?", Array As String(contactID), "")
	Dim resultUri As String = ""
	If contentCursor.RowCount > 0 Then
		contentCursor.Position = 0
		resultUri = contentCursor.GetString("photo_uri")
	End If
	Return resultUri
End Sub

'This will return true if the given contact data is a primary one of the contact, otherwise false will be returned.
Public Sub GetPrimary(contactID As Long, mimeType As String, data() As Object) As Boolean
	Dim selectionItems As List
	Dim whereBuilder As StringBuilder
	selectionItems.Initialize
	selectionItems.Add(mimeType)
	selectionItems.Add(contactID)
	whereBuilder.Initialize
	whereBuilder.Append("mimetype = ? AND contact_id = ?")
	If data <> Null And data.Length > 0 Then
		For index = 1 To data.Length
			Dim dataValue As String = data(index - 1)
			If dataValue.Length > 0 And dataValue.EqualsIgnoreCase("null") = False Then
				selectionItems.Add(dataValue)
				whereBuilder.Append(" AND data").Append(index).Append(" = ?")
			End If
		Next
	End If
	Dim selections(selectionItems.Size) As String
	For index = 0 To selectionItems.Size - 1
		selections(index) = selectionItems.Get(index)
	Next
	Dim primaryCursor As Cursor = internalContentResolver.Query(internalDataUri, Array As String("is_primary", _ 
																"is_super_primary"), whereBuilder.ToString, selections, "")
	Dim result As Boolean = False
	If primaryCursor.RowCount > 0 Then
		primaryCursor.Position = 0
		'result = primaryCursor.GetInt("is_primary") = 1
		result = primaryCursor.GetInt("is_super_primary") = 1
	End If
	primaryCursor.Close
	Return result
End Sub

'This will return true if the given contact id was starred, otherwise false will be returned.
Public Sub GetStarred(contactID As Long) As Boolean
	Dim starredCursor As Cursor = internalContentResolver.Query(internalContactsUri, Array As String("starred"), _
																"_id = ?", Array As String(contactID), "")
	starredCursor.Position = 0
	Dim result As Boolean = starredCursor.GetInt("starred") = 1
	starredCursor.Close
	Return result
End Sub

'This will return the max display photo dimensions.
Public Sub getMaxDisplayPhotoDimensions As Int
	Return internalDisplayPhotoDimensions
End Sub

'This will return the max thumbnail photo dimensions.
Public Sub getMaxThumbnailPhotoDimensions As Int
	Return internalThumbnailPhotoDimensions
End Sub

Public Sub SetData(contactID As Long, mimeType As String, dataValues() As Object, convertExact As Boolean) As Boolean
	Dim values As ContentValues
	values.Initialize
	If dataValues <> Null And dataValues.Length > 0 Then
		For index = 1 To dataValues.Length
			Dim keyValue As String = "data" & index
			Dim dataValue As Object = dataValues(index - 1)
			If dataValue = Null Then
				If convertExact Then
					values.PutNull(keyValue)
				End If
			Else If dataValue Is String Then
				Dim dataString As String = dataValue
				If dataString.Length > 0 Then
					If dataString.EqualsIgnoreCase("null") Then
						If convertExact Then
							values.PutNull(keyValue)
						End If
					Else
						values.PutString(keyValue, dataValue)		
					End If
				Else
					If convertExact Then
						values.PutString(keyValue, dataValue)
					'Else
						'values.PutNull(keyValue)
					End If
				End If
			Else If dataValue Is Int Then
				values.PutInteger(keyValue, dataValue)
			Else If dataValue Is Long Then
				values.PutLong(keyValue, dataValue)
			Else If dataValue Is Boolean Then
				values.PutBoolean(keyValue, dataValue)
			End If
		Next
	End If
	Return SetData2(contactID, mimeType, values, False)
End Sub

Private Sub GetRawContactID(contactID As Long) As Long
	Dim resultRawContactID As Long = 0
	Dim contactCursor As Cursor
	contactCursor = internalContentResolver.Query(internalRawContactsUri, Array As String("_id"), _
	 											  "contact_id = ?", Array As String(contactID), "")
	If contactCursor.RowCount > 0 Then
		contactCursor.Position = 0
		resultRawContactID = contactCursor.GetLong("_id")
	Else
		contactCursor.Close
		contactCursor = internalContentResolver.Query(internalContactsUri, Array As String("name_raw_contact"), _ 
													  "_id = ?", Array As String(contactID), "")
		If contactCursor.RowCount > 0 Then
			contactCursor.Position = 0
			resultRawContactID = contactCursor.GetLong("name_raw_contact")
		End If
	End If
	contactCursor.Close
	Return resultRawContactID
End Sub

Public Sub SetData2(contactID As Long, mimeType As String, values As ContentValues, update As Boolean) As Boolean
	Dim resultDone As Boolean = True
	If update Then
		internalContentResolver.Update(internalDataUri, values, "mimetype = ? AND contact_id = ?", Array As String(mimeType, contactID))
	Else
'		Dim contactCursor As Cursor = internalContentResolver.Query(internalRawContactsUri, Array As String("_id"), _
'	 															"contact_id = ?", Array As String(contactID), "")
'		
''		Dim contactCursor As Cursor = internalContentResolver.Query(internalContactsUri, Array As String("name_raw_contact_id"), _
''																	"_id = ?", Array As String(contactID), "")
'		If contactCursor.RowCount = 0 Then
'			Log("Error getting raw_contact_id")
'			contactCursor.Close
'			Return False
'		End If
'		contactCursor.Position = 0
'		'values.PutString("raw_contact_id", contactCursor.GetString("name_raw_contact_id"))
'		values.PutString("raw_contact_id", contactCursor.GetString("_id"))
'		contactCursor.Close
		Dim rawContactID As Long = GetRawContactID(contactID)
		If rawContactID = 0 Then
			LogColor("ContactHandler->SetData2: Error getting raw_contact_id!", Colors.Red)
			resultDone = False
		Else
			values.PutString("raw_contact_id", rawContactID)
			values.PutString("mimetype", mimeType)
			internalContentResolver.Insert(internalDataUri, values)
		End If
	End If
	Return resultDone
End Sub

'This will set the given thumbnail photo to the contact that was found by the given contact id.
'- The thumbnail photo will be resized if the height and/or the width was bigger than the system
'  defined thumbnail photo dimensions.
Public Sub SetThumbnailPhoto(contactID As Long, thumbnailPhoto As Bitmap)
	'First check out if there is a thumbnail photo already
	Dim update As Boolean = False
	Dim photoData As List = GetData(contactID, ContactMimeTypes.PHOTO, Array As String("data15"), Array As Boolean(True))
	If photoData.Size > 0 Then
		update = True
		photoData.Clear
	End If
	
	'If the photo is to big, try to resize it
	Dim maxThumbnailDimensions As Int = getMaxThumbnailPhotoDimensions
	If thumbnailPhoto.Height > maxThumbnailDimensions Or thumbnailPhoto.Width > maxThumbnailDimensions Then
		Dim tempOutputStream As OutputStream = File.OpenOutput(File.DirInternalCache, "temp0001", False)
		thumbnailPhoto.WriteToStream(tempOutputStream, 100, "PNG")
		tempOutputStream.Close
		Dim newThumbnailPhoto As Bitmap
		newThumbnailPhoto.InitializeSample(File.DirInternalCache, "temp0001", maxThumbnailDimensions, maxThumbnailDimensions)
		File.Delete(File.DirInternalCache, "temp0001")
		thumbnailPhoto = newThumbnailPhoto
	End If
	
	'Write the photo into the memory and set it to the database
	Dim memoryOutputStream As OutputStream
	memoryOutputStream.InitializeToBytesArray(1)
	thumbnailPhoto.WriteToStream(memoryOutputStream, 100, "PNG")
	Dim buffer() As Byte = memoryOutputStream.ToBytesArray
	memoryOutputStream.Close
	Dim values As ContentValues
	values.Initialize
	values.PutBytes("data15", buffer)
	SetData2(contactID, ContactMimeTypes.PHOTO, values, update)
End Sub

'This will set the given contact data as a primary one of the contact or as not a primary one.
Public Sub SetPrimary(contactID As Long, mimeType As String, data() As Object, primary As Boolean)
	Dim selectionItems As List
	Dim whereBuilder As StringBuilder
	selectionItems.Initialize
	selectionItems.Add(mimeType)
	selectionItems.Add(contactID)
	whereBuilder.Initialize
	whereBuilder.Append("mimetype = ? AND contact_id = ?")
	If data <> Null And data.Length > 0 Then
		For index = 1 To data.Length
			Dim dataValue As String = data(index - 1)
			If dataValue.Length > 0 And dataValue.EqualsIgnoreCase("null") = False Then
				selectionItems.Add(dataValue)
				whereBuilder.Append(" AND data").Append(index).Append(" = ?")
			End If
		Next
	End If
	Dim selections(selectionItems.Size) As String
	For index = 0 To selectionItems.Size - 1
		selections(index) = selectionItems.Get(index)
	Next
	Dim primaryValue As Int = 0
	If primary Then
		primaryValue = 1
	End If
	Dim values As ContentValues
	values.Initialize
	values.PutInteger("is_super_primary", primaryValue)
	internalContentResolver.Update(internalDataUri, values, whereBuilder.ToString, selections)
End Sub

'This will set the contact that was found by the given contact id to starred or unstarred state.
Public Sub SetStarred(id As Long, starred As Boolean)
	Dim values As ContentValues
	values.Initialize
	values.PutBoolean("starred", starred)
	internalContentResolver.Update(internalContactsUri, values, "_id = ?", Array As String(id))
End Sub

'This will update the given contacts mimetype data with the given data values.
Public Sub UpdateData(contactID As Long, mimeType As String, dataValues() As Object, convertExact As Boolean)
	Dim values As ContentValues
	values.Initialize
	If dataValues <> Null And dataValues.Length > 0 Then
		For index = 1 To dataValues.Length
			Dim keyValue As String = "data" & index
			Dim dataValue As Object = dataValues(index - 1)
			If dataValue = Null Then
				If convertExact Then
					values.PutNull(keyValue)
				End If
			Else If dataValue Is String Then
				Dim dataString As String = dataValue
				If dataString.Length > 0 Then
					If dataString.EqualsIgnoreCase("null") Then
						If convertExact Then
							values.PutNull(keyValue)
						End If
					Else
						values.PutString(keyValue, dataValue)		
					End If
				Else
					If convertExact Then
						values.PutString(keyValue, dataValue)
					'Else
						'values.PutNull(keyValue)
					End If
				End If
			Else If dataValue Is Int Then
				values.PutInteger(keyValue, dataValue)
			Else If dataValue Is Long Then
				values.PutLong(keyValue, dataValue)
			Else If dataValue Is Boolean Then
				values.PutBoolean(keyValue, dataValue)
			End If
		Next
	End If
	SetData2(contactID, mimeType, values, True) 
End Sub