﻿Type=StaticCode
Version=6
ModulesStructureVersion=1
B4A=true
@EndOfDesignText@
'ContactStatics Code module
Private Sub Process_Globals
	Public Const CONTACTS_URL As String = "content://com.android.contacts/contacts"
	Public Const DATA_URL As String = "content://com.android.contacts/data"
	Public Const GROUPS_URL As String = "content://com.android.contacts/groups"
	Public Const RAW_CONTACTS_URL As String = "content://com.android.contacts/raw_contacts"
	
	Private internalAddressTypes As Map
	Private internalEmailAddressTypes As Map
	Private internalEventTypes As Map
	Private internalFullNameStyles As Map
	Private internalIMProtocols As Map
	Private internalIMTypes As Map
	Private internalNicknameTypes As Map
	Private internalOrganizationTypes As Map
	Private internalPhoneNumberTypes As Map
	Private internalRelationTypes As Map
	Private internalSipAddressTypes As Map
	Private internalWebsiteTypes As Map
End Sub

'Get structured postal/ address types map. 
'- Use this map only for reading, it will be generated on the first access only.
Public Sub AddressTypes As Map
	If internalAddressTypes.IsInitialized = False Then
		internalAddressTypes.Initialize
		internalAddressTypes.Put("0", "custom") 'LABEL must be set
		internalAddressTypes.Put("1", "home")
		internalAddressTypes.Put("2", "work")
		internalAddressTypes.Put("3", "other")
	End If
	Return internalAddressTypes
End Sub

'This will return the email address types map with all valid email address type values.
'- Use this map only for reading, it will be generated on the first access only.
Public Sub EmailAddressTypes As Map
	If internalEmailAddressTypes.IsInitialized = False Then
		internalEmailAddressTypes.Initialize
		internalEmailAddressTypes.Put("0", "custom") 'LABEL must be set
		internalEmailAddressTypes.Put("1", "home")
		internalEmailAddressTypes.Put("2", "work")
		internalEmailAddressTypes.Put("3", "other")
		internalEmailAddressTypes.Put("4", "mobile")
	End If
	Return internalEmailAddressTypes
End Sub

'This will return the event types map with all valid event type values. 
'- Use this map only for reading, it will be generated on the first access only.
Public Sub EventTypes As Map
	If internalEventTypes.IsInitialized = False Then
		internalEventTypes.Initialize
		internalEventTypes.Put("0", "custom") 'LABEL must be set
		internalEventTypes.Put("1", "anniversary")
		internalEventTypes.Put("2", "other")
		internalEventTypes.Put("3", "birthday")
	End If
	Return internalEventTypes
End Sub

'This will return the full name styles map with all valid full name style values.
'- Use this map only for reading, it will be generated on the first access only.
Public Sub FullNameStyles As Map
	If internalFullNameStyles.IsInitialized = False Then
		internalFullNameStyles.Initialize
		internalFullNameStyles.Put("0", "undefined")
		internalFullNameStyles.Put("1", "western")
		internalFullNameStyles.Put("2", "cjk")
		internalFullNameStyles.Put("3", "chinese")
		internalFullNameStyles.Put("4", "japanese")
		internalFullNameStyles.Put("5", "korean")
	End If
	Return internalFullNameStyles
End Sub

'This is a helper function to get the key of the given value, otherwise the default key will be returned.
Public Sub GetKeyFromValue(map As Map, value As String, defaultKey As Int) As Int
	Dim resultKey As Int = defaultKey
	For index = 0 To map.Size - 1
		If map.GetValueAt(index) = value Then
			resultKey = map.GetKeyAt(index)
			Exit
		End If
	Next
	Return resultKey
End Sub

Public Sub HasValue(map As Map, value As String) As Boolean
	Dim result As Boolean = False
	For Each mapValue As String In map.Values
		If mapValue.EqualsIgnoreCase(value) Then
			result = True
			Exit
		End If
	Next
	Return result
End Sub

'This will return the im (= instant messanger) protocols map with all valid protocol values.
'- Use this map only for reading, it will be generated on the first access only.
Public Sub IMProtocols As Map
	If internalIMProtocols.IsInitialized = False Then
		internalIMProtocols.Initialize
		internalIMProtocols.Put("-1", "custom") 'CUSTOMPROTOCOL must be set
		internalIMProtocols.Put("0", "aim")
		internalIMProtocols.Put("1", "msn")
		internalIMProtocols.Put("2", "yahoo")
		internalIMProtocols.Put("3", "skype")
		internalIMProtocols.Put("4", "qq")
		internalIMProtocols.Put("5", "google_talk")
		internalIMProtocols.Put("6", "icq")
		internalIMProtocols.Put("7", "jabber")
		internalIMProtocols.Put("8", "netmeeting")
	End If
	Return internalIMProtocols
End Sub

'This will return the im (= instant messanger) types map with all valid type values.
'- Use this map only for reading, it will be generated on the first access only.
Public Sub IMTypes As Map
	If internalIMTypes.IsInitialized = False Then
		internalIMTypes.Initialize
		internalIMTypes.Put("0", "custom") 'LABEL must be set
		internalIMTypes.Put("1", "home")
		internalIMTypes.Put("2", "work")
		internalIMTypes.Put("3", "other")
	End If
	Return internalIMTypes
End Sub

'This will return the nickname types map with all valid nickname type values.
'- Use this map only for reading, it will be generated on the first access only.
Public Sub NicknameTypes As Map
	If internalNicknameTypes.IsInitialized = False Then
		internalNicknameTypes.Initialize
		internalNicknameTypes.Put("0", "custom") 'LABEL must be set
		internalNicknameTypes.Put("1", "default")
		internalNicknameTypes.Put("2", "other_name")
		internalNicknameTypes.Put("3", "maiden_name")
		internalNicknameTypes.Put("4", "short_name")
		internalNicknameTypes.Put("5", "initials")
	End If
	Return internalNicknameTypes
End Sub

'This will return the organization types map with all valid organization type values.
'- Use this map only for reading, it will be generated on the first access only.
Public Sub OrganizationTypes As Map
	If internalOrganizationTypes.IsInitialized = False Then
		internalOrganizationTypes.Initialize
		internalOrganizationTypes.Put("0", "custom") 'LABEL must be set
		internalOrganizationTypes.Put("1", "work")
		internalOrganizationTypes.Put("2", "other")
	End If
	Return internalOrganizationTypes
End Sub

'This will return the phone number types map with all valid phone number type values.
'- Use this map only for reading, it will be generated on the first access only.
Public Sub PhoneNumberTypes As Map
	If internalPhoneNumberTypes.IsInitialized = False Then
		internalPhoneNumberTypes.Initialize
		internalPhoneNumberTypes.Put("0", "custom") 'LABEL must be set
		internalPhoneNumberTypes.Put("1", "home")
		internalPhoneNumberTypes.Put("2", "mobile")
		internalPhoneNumberTypes.Put("3", "work")
		internalPhoneNumberTypes.Put("4", "fax_work")
		internalPhoneNumberTypes.Put("5", "fax_home")
		internalPhoneNumberTypes.Put("6", "pager")
		internalPhoneNumberTypes.Put("7", "other")
		internalPhoneNumberTypes.Put("8", "callback")
		internalPhoneNumberTypes.Put("9", "car")
		internalPhoneNumberTypes.Put("10", "company_main")
		internalPhoneNumberTypes.Put("11", "isdn")
		internalPhoneNumberTypes.Put("12", "main")
		internalPhoneNumberTypes.Put("13", "other_fax")
		internalPhoneNumberTypes.Put("14", "radio")
		internalPhoneNumberTypes.Put("15", "telex")
		internalPhoneNumberTypes.Put("16", "tty_tdd")
		internalPhoneNumberTypes.Put("17", "work_mobile")
		internalPhoneNumberTypes.Put("18", "work_pager")
		internalPhoneNumberTypes.Put("19", "assistant")
		internalPhoneNumberTypes.Put("20", "mms")
	End If
	Return internalPhoneNumberTypes
End Sub

'This will return the relation types map with all valid relation type values.
'- Use this map only for reading, it will be generated on the first access only.
Public Sub RelationTypes As Map
	If internalRelationTypes.IsInitialized = False Then
		internalRelationTypes.Initialize
		internalRelationTypes.Put("0", "custom") 'LABEL must be set
		internalRelationTypes.Put("1", "assistant")
		internalRelationTypes.Put("2", "brother")
		internalRelationTypes.Put("3", "child")
		internalRelationTypes.Put("4", "domestic_partner")
		internalRelationTypes.Put("5", "father")
		internalRelationTypes.Put("6", "friend")
		internalRelationTypes.Put("7", "manager")
		internalRelationTypes.Put("8", "mother")
		internalRelationTypes.Put("9", "parent")
		internalRelationTypes.Put("10", "partner")
		internalRelationTypes.Put("11", "referred_by")
		internalRelationTypes.Put("12", "relative")
		internalRelationTypes.Put("13", "sister")
		internalRelationTypes.Put("14", "spouse")
	End If
	Return internalRelationTypes
End Sub

'This will return the sip address types map with all valid sip address type value.
'- Use this map only for reading, it will be generated on the first access only.
Public Sub SipAddressTypes As Map
	If internalSipAddressTypes.IsInitialized = False Then
		internalSipAddressTypes.Initialize
		internalSipAddressTypes.Put("0", "custom") 'LABEL must be set
		internalSipAddressTypes.Put("1", "home")
		internalSipAddressTypes.Put("2", "work")
		internalSipAddressTypes.Put("3", "other")
	End If
	Return internalSipAddressTypes
End Sub

'This will return the website types map with all valid website type values.
'- Use this map only for reading, it will be generated on the first access only.
Public Sub WebsiteTypes As Map
	If internalWebsiteTypes.IsInitialized = False Then
		internalWebsiteTypes.Initialize
		internalWebsiteTypes.Put("0", "custom") 'LABEL must be set
		internalWebsiteTypes.Put("1", "homepage")
		internalWebsiteTypes.Put("2", "blog")
		internalWebsiteTypes.Put("3", "profile")
		internalWebsiteTypes.Put("4", "home")
		internalWebsiteTypes.Put("5", "work")
		internalWebsiteTypes.Put("6", "ftp")
		internalWebsiteTypes.Put("7", "other")
	End If
	Return internalWebsiteTypes
End Sub